#include <Adafruit_NeoPixel.h>
#include <avr/power.h>

// Which pin on the Arduino is connected to the NeoPixels?
// On a Trinket or Gemma we suggest changing this to 1
#define PIN            9

// Right D4
// Left D5
// Down D6
// Up D7
// Fire D2

// How many NeoPixels are attached to the Arduino?
#define NUMPIXELS      247


// When we setup the NeoPixel library, we tell it how many pixels, and which pin to use to send signals.
// Note that for older NeoPixel strips you might need to change the third parameter--see the strandtest
// example for more information on possible values.
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

byte Snake[NUMPIXELS]; // array for snake, maximum length is 247, it mean all video :)
byte SnakeHead, SnakeEnd; // position in array
byte Food; // pixel with food
byte Direction; // move direction
bool Growe; // indicate, that snake will growe
byte State = 0; // indicate, state (0=alive,1=hit wall,2=self eat)
byte TextPos = 0;
byte MyStep;

const PROGMEM byte HW[25] = {              // head to the wall text
                  B00000000,
                  B00000000,
                  B01111000,
                  B00010100,
                  B01111000,
                  B00000000,
                  B01111000,
                  B01000000,
                  B01111000,
                  B00000000,
                  B00000000,
                  B00000000,
                  B01101000,
                  B01011000,
                  B01001000,
                  B00000000,
                  B00110000,
                  B01011000,
                  B01010000,
                  B00000010,
                  B00100100,
                  B01010010,
                  B01111100,
                  B00000000,
                  B00000000};
                  
const PROGMEM byte SelfEat[53] = {              // self eat text
                  B00000000,
                  B00000000,
                  B01111000,
                  B00010100,
                  B01111000,
                  B00000000,
                  B01111000,
                  B01000000,
                  B01111000,
                  B00000000,
                  B00000000,
                  B00000000,
                  B01010000,
                  B01011000,
                  B00101000,
                  B00000000,
                  B00110000,
                  B01011000,
                  B01010000,
                  B00000000,
                  B01111000,
                  B00011000,
                  B01111000,
                  B00000000,
                  B00000000,
                  B00000000,
                  B01010000,
                  B01011000,
                  B00101000,
                  B00000000,
                  B00110000,
                  B01011000,
                  B01010000,
                  B00000000,
                  B00000000,
                  B00000000,
                  B01111100,
                  B00100000,
                  B01010000,
                  B00000000,
                  B00110000,
                  B01001000,
                  B00110000,
                  B00000000,
                  B01111000,
                  B01000000,
                  B01111000,
                  B00000000,
                  B01010000,
                  B01011000,
                  B00101000,
                  B00000000,
                  B00000000};

void cls() {
  for(int i=0;i<(NUMPIXELS);i++){
    pixels.setPixelColor(i, 0); // Black color.
  }
  //ReDraw = 1;
}

uint8_t compress (uint8_t X, uint8_t Y) {
  return (Y * 19) + X;
  }

uint8_t PosX (uint8_t C) {
  return C % 19;
  }

uint8_t PosY (uint8_t C) {
  return C / 19;
  }

void setup() {
  // This is for Trinket 5V 16MHz, you can remove these three lines if you are not using a Trinket
#if defined (__AVR_ATtiny85__)
  if (F_CPU == 16000000) clock_prescale_set(clock_div_1);
#endif
  // End of trinket special code
  pinMode(2, INPUT); // fire button
  pixels.begin(); // This initializes the NeoPixel library.
  randomSeed(analogRead(5));
  random(7);
  NewGame();
}
void NewGame() {
  cls();
  SnakeHead = 2;
  SnakeEnd = 0;
  Snake[0] = compress(3,3);
  pixels.setPixelColor(compress(3,3), 0,0,20); // Dark blue color.
  Snake[1] = compress(3,4);
  pixels.setPixelColor(compress(3,4), 0,0,20); // Dark blue color.
  Snake[2] = compress(3,5);
  pixels.setPixelColor(compress(3,5), 0,0,20); // Dark blue color.
  pixels.show();
  NewFood();
  Direction = 16;  
  State = 0;
  TextPos=0;
  MyStep=0;
}

void MoveHead(void) {
// Right D4 (16)
// Left D5 (32)
// Down D6 (64)
// Up D7 (128)
// Fire D2 (4)
  byte Keyboard = (PIND & 0xF0); // read keyboard
  byte Head = Snake[SnakeHead];
  byte NewHead;
  pixels.setPixelColor(Head, 0,0,20); // Dark blue color.
  if (Keyboard) {Direction=Keyboard;}
  if (Direction & 16)
  {
    // right
    if (PosX(Head) < 18) {
      // step
      NewHead = Head + 1;
      Growe = (NewHead == Food);
      if ((Growe) || (!pixels.getPixelColor(NewHead))) {
        SnakeHead++;
        if (SnakeHead == NUMPIXELS) {SnakeHead = 0;}
        Snake[SnakeHead]=NewHead;
        pixels.setPixelColor(NewHead, 0,0,40); // Light blue color.
        //Serial.print("x="); Serial.print(NewHead >> 4); Serial.print("y="); Serial.println(NewHead & 0x0F);
      }
      else
      {
        // self
        State = 2;
      }
    }
    else
    {
      // head to wall
      State = 1;
    }
  }
  else if (Direction & 32)
  {
    // left
    if (PosX(Head) > 0) {
      // step
      NewHead = Head - 1;
      Growe = (NewHead == Food);
      if ((Growe) || (!pixels.getPixelColor(NewHead))) {
        SnakeHead++;
        if (SnakeHead == NUMPIXELS) {SnakeHead = 0;}
        Snake[SnakeHead]=NewHead;
        pixels.setPixelColor(NewHead, 0,0,40); // Light blue color.
        //Serial.print("x="); Serial.print(NewHead >> 4); Serial.print("y="); Serial.println(NewHead & 0x0F);
      }
      else
      {
        // self
        State = 2;
      }
    }
    else
    {
      // head to wall
      State = 1;
    }
  }
  else if (Direction & 128)
  {
    // up
    if (PosY(Head) > 0) {
      // step
      NewHead = Head - 19;
      Growe = (NewHead == Food);
      if ((Growe) || (!pixels.getPixelColor(NewHead))) {
        SnakeHead++;
        if (SnakeHead == NUMPIXELS) {SnakeHead = 0;}
        Snake[SnakeHead]=NewHead;
        pixels.setPixelColor(NewHead, 0,0,40); // Light blue color.
        //Serial.print("x="); Serial.print(NewHead >> 4); Serial.print("y="); Serial.println(NewHead & 0x0F);
      }
      else
      {
        // self
        State = 2;
      }
    }
    else
    {
      // head to wall
      State = 1;
    }
  }
  else
  {
    // down
    if (PosY(Head) < 12) {
      // step
      NewHead = Head + 19;
      Growe = (NewHead == Food);
      if ((Growe) || (!pixels.getPixelColor(NewHead))) {
        SnakeHead++;
        if (SnakeHead == NUMPIXELS) {SnakeHead = 0;}
        Snake[SnakeHead]=NewHead;
        pixels.setPixelColor(NewHead, 0,0,40); // Light blue color.
        //Serial.print("x="); Serial.print(NewHead >> 4); Serial.print("y="); Serial.println(NewHead & 0x0F);
      }
      else
      {
        // self
        State = 2;
      }
    }
    else
    {
      // head to wall
      State = 1;
    }
  }

  //}
}

void MoveTail()
{
  pixels.setPixelColor(Snake[SnakeEnd], 0); // black color.
  SnakeEnd++;
  if (SnakeEnd == NUMPIXELS) {SnakeEnd = 0;}
}

void NewFood()
{
  do
  {
    Food = compress(random(19), random(13));
  } while (pixels.getPixelColor(Food));
  pixels.setPixelColor(Food, 20,0,0); // red color.
}

void NextHeadToWall()
{
  byte i;
  for(i=0;i<NUMPIXELS-1;i++) {pixels.setPixelColor(i,pixels.getPixelColor(i+1));}
  byte j=1;
  byte HeadToWall = pgm_read_byte(HW + TextPos);
  for(i=0;i<7;i++) {
    if (HeadToWall & j)
    {
      pixels.setPixelColor(compress(18,i), 0,20,0); // green color.
      //vid.video[i]=(vid.video[i] << 1) + 1;
    }
    else
    {
      pixels.setPixelColor(compress(18,i), 0,0,0); // black color.
      //vid.video[i]=(vid.video[i] << 1);
    }
    j = j << 1;
  }
  for(i=7;i<13;i++) {pixels.setPixelColor(compress(18,i), 0,0,0);} // black color.
  TextPos++;
  if (TextPos>=sizeof(HW)) {TextPos=0;}
  if (digitalRead(2) == HIGH) {NewGame();}
}

void NextSelfEat() 
{
  byte i;
  for(i=0;i<NUMPIXELS-1;i++) {pixels.setPixelColor(i,pixels.getPixelColor(i+1));}
  byte j=1;
  byte SelfEatB = pgm_read_byte(SelfEat + TextPos);
  for(i=0;i<7;i++) {
    if (SelfEatB & j)
    {
      pixels.setPixelColor(compress(18,i), 0,20,0); // green color.
      //vid.video[i]=(vid.video[i] << 1) + 1;
    }
    else
    {
      pixels.setPixelColor(compress(18,i), 0,0,0); // black color.
      //vid.video[i]=(vid.video[i] << 1);
    }
    j = j << 1;
  }
  for(i=7;i<13;i++) {pixels.setPixelColor(compress(18,i), 0,0,0);} // black color.
  TextPos++;
  if (TextPos>=sizeof(SelfEat)) {TextPos=0;}
  if (digitalRead(2) == HIGH) {NewGame();}
}

void loop() {
  byte Keyboard = (PIND & 0xF0); // read keyboard
  if (Keyboard) {Direction=Keyboard;}
  if (MyStep > 20) {
    MyStep = 0;
    switch (State) {
      case 0:
        // normal alive
        MoveHead();
        if (Growe) {NewFood();} else {MoveTail();}
        break;
      case 1:
        // head to the wall
        NextHeadToWall();
        break;
      case 2:
        // self eating
        NextSelfEat();
        break;
    }
    pixels.show();
  }
  MyStep++;
  delay(10);
}

