// Clear screen \033[2J
// Go to position \033[1;1H
// Hide cursor \033[?25l
// UnHide cursor \033[?25h

// Set color (30+x foreground, 40+x background) \033[31;1m
// ; separate next command for color ;1 = bright, ;0 reset
// colors: 0-black, 1-red, 2 green, 3 yellow, 4 blue, 
//         5 Magenta, 6 cyan, 7 white

// PINS:
// PORTD
// PD0/D0 = RxD
// PD1/D1 = TxD
// PD2/D2
// PD3/D3
// PD4/D4
// PD5/D5
// PD6/D6
// PD7/D7 = NeoPixel
// PORTB
// PB0/D8
// PB1/D9
// PB2/D10 = SS (SD CS)
// PB3/D11 = MOSI (SD MOSI)
// PB4/D12 = MISO (SD MISO)
// PB5/D13 = CSK (SD SCK)

#include <Fat16.h>
#include <Fat16util.h> // use functions to print strings from flash memory
#include <Adafruit_NeoPixel.h>
#include <avr/power.h>

const PROGMEM char font1[] = {
0,0,0, //' '
0,95,0, //'!'
62,20,62, //'#'
38,107,50, //'$'
97,28,67, //'%'
54,41,118, //'&'
0,5,3, //"'"
28,34,65, //'('
65,34,28, //')'
42,28,42, //'*'
8,62,8, //'+'
0,80,48, //','
8,8,8, //'-'
0,96,0, //'.'
96,28,3, //'/'
62,77,62, //'0'
66,127,64, //'1'
114,73,70, //'2'
34,73,54, //'3'
14,8,127, //'4'
39,73,49, //'5'
62,73,50, //'6'
97,25,7, //'7'
54,73,54, //'8'
38,73,62, //'9'
0,54,0, //':'
64,52,0, //';'
8,20,99, //'<'
20,20,20, //'='
99,20,8, //'>'
2,89,6, //'?'
60,90,92, //'@'
126,9,126, //'A'
127,73,54, //'B'
62,65,34, //'C'
127,65,62, //'D'
127,73,65, //'E'
127,9,1, //'F'
62,73,58, //'G'
127,8,127, //'H'
65,127,65, //'I'
32,64,63, //'J'
127,8,119, //'K'
127,64,64, //'L'
127,6,127, //'M'
126,12,127, //'N'
62,65,62, //'O'
127,9,6, //'P'
62,65,190, //'Q'
127,9,118, //'R'
38,73,50, //'S'
1,127,1, //'T'
127,64,127, //'U'
31,96,31, //'V'
127,48,127, //'W'
119,28,119, //'X'
7,120,7, //'Y'
113,73,71, //'Z'
0,127,65, //'['
6,24,96, //'\'
65,127,0, //']'
6,3,6, //'^'
128,128,128, //'_'
3,5,0, //'`'
32,84,120, //'a'
127,68,56, //'b'
56,68,40, //'c'
56,68,127, //'d'
56,84,88, //'e'
8,126,9, //'f'
152,164,124, //'g'
127,4,120, //'h'
4,125,0, //'i'
64,128,125, //'j'
127,16,108, //'k'
65,127,64, //'l'
124,8,124, //'m'
124,4,120, //'n'
56,68,56, //'o'
252,36,24, //'p'
24,36,252, //'q'
124,8,4, //'r'
72,84,36, //'s'
4,62,68, //'t'
60,64,124, //'u'
28,96,28, //'v'
124,32,124, //'w'
108,16,108, //'x'
76,80,60, //'y'
100,84,76, //'z'
8,119,65, //'{'
0,127,0, //'|'
65,119,8, //'}'
2,1,3, //'~'
28,20,28 //''
}; 

// Which pin on the Arduino is connected to the NeoPixels?
// On a Trinket or Gemma we suggest changing this to 1
#define PIXELPIN            9
// How many NeoPixels are attached to the Arduino?
#define NUMPIXELS      247
// When we setup the NeoPixel library, we tell it how many pixels, and which pin to use to send signals.
// Note that for older NeoPixel strips you might need to change the third parameter--see the strandtest
// example for more information on possible values.
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, PIXELPIN, NEO_GRB + NEO_KHZ800);


#define MAX_X 19
#define MAX_Y 13

const uint8_t CHIP_SELECT = SS;  // SD card chip select pin.
SdCard card;
Fat16 file;

int PosX, PosY;
int MaxX, MaxY;
int incomingByte;
int Steps, Pushes;
int Boxes, BoPs;
byte Draw;
byte Level=1;
byte i,j;
byte GameState=0; // 0=select level (+Load) 1=play

char level[MAX_Y][MAX_X+1];

// store error strings in flash to save RAM
#define error(s) error_P(PSTR(s))

void error_P(const char* str) {
  PgmPrint("error: ");
  SerialPrintln_P(str);
  if (card.errorCode) {
    PgmPrint("SD error: ");
    Serial.println(card.errorCode, HEX);
  } 
  while(1);
}

void ReadLevel(byte LevelNo) {
  char line[50];
  int n;
  byte x=0;
  byte y=0;
  byte found = 0;
  char *p, *p2;
  byte tx,ty;

  //itoa(LevelNo,chLevelNo,10);
    // open a file
  if (!file.open("SOKOBAN.XML", O_READ)) {
    error("file.open");
  }
  while (((n = file.fgets(line, sizeof(line))) > 0) && (found<2)){
    //Serial.println(line);
    switch (found) {
      case 0 :
        p = strstr(line,"<Level ");
        if (p) {
          LevelNo--;
          //Serial.print(F("Level "));
          //Serial.println(LevelNo);
          if (LevelNo==0) {
              p2= strstr(p,"Width=\"");
              if (p2) {
                p2+=7;
                while ((*p2>='0') && (*p2<='9')) {x*=10;x+=(*p2-'0');p2++;}
              }
              p2= strstr(p,"Height=\"");
              if (p2) {
                p2+=8;
                while ((*p2>='0') && (*p2<='9')) {y*=10;y+=(*p2-'0');p2++;}
              }
              //Serial.print(F("MaxX="));
              //Serial.print(x);
              //Serial.print(F(" MaxY="));
              //Serial.println(y);
              if ((x==0) || (x>MAX_X) || (y==0) || (y>MAX_Y)) {
                LevelNo=1;
                x=0;
                y=0;
              } else {found=1;ty=0;}
            }
          }
          break;
      case 1 :
        p = strstr(line,"<L>");
        if (p) {
          //Serial.print(F("L "));
          //Serial.println(ty);
          tx=0;
          p+=3;
          while ((*p!='<') &&(tx<x)) {level[ty][tx++]=*p;p++;}
          while (tx<x) {level[ty][tx++]=' ';}
          ty++;
          if (ty==y) {found=2;}
        }
        break;
    }
  }
  file.close();
  if (found<2) {
    x=5;y=3;
    level[0][0]=level[0][1]=level[0][2]=level[0][3]=level[0][4]='#';
    level[1][0]='#';level[1][1]='@';level[1][2]='$';level[1][3]='.';level[1][4]='#';
    level[2][0]=level[2][1]=level[2][2]=level[2][3]=level[2][4]='#';
  }
  MaxX=x;
  MaxY=y;
  for (i = 0; i < MaxY; i++){
    for (j = 0; j < MaxX; j++){
     if ((level[i][j] == '@') || (level[i][j] == '+')) {PosX=j; PosY=i;};
     }
   }
  Steps=0; Pushes=0;
  Boxes=0; BoPs=0;
  //Serial.print(F("FreeMem:"));
  //Serial.println(FreeRam());

}

void cls2() {
  Serial.print(F("\033[0m\033[2J\033[1;1H"));
  // \033[0m   - go to column 0
  // \033[2J   - Erase in Display ( 2 = all)
  // \033[1;1H - Cursor Position [row;column] (1,1)
  //Serial.print(F("\033[1;1H\033[0m\033[?25l"));
  //Serial.print("\033[1;1H\033[0J\033[0m");
  //pixels.clear(); // erase pixels display
}

void cls() {
  //Serial.print("\033[2J\033[1;1H\033[0m");
  Serial.print(F("\033[1;1H\033[0m\033[?25l"));
  // \033[1;1H - Cursor Position [row;column] (1,1)
  // \033[0m   - go to column 0
  // \033[?25l - Hide Cursor (DECTCEM) 
  //Serial.print("\033[1;1H\033[0J\033[0m");
  pixels.clear(); // erase pixels display
}

char ConvChar(const char source) {
  if ((source<33) || (source>127)) { return 94; }
  else { return source - 33; }
}

void LnPrint(const uint32_t c,const char* mystr) {
byte i,j,k,m;
  m=1;
  for(i=0;i<8;i++){
    for(j=0;j<(NUMPIXELS-19);j++) {
      pixels.setPixelColor(j,pixels.getPixelColor(19+j)); // one line up
    }
    for(;j<(NUMPIXELS);j++) {
      pixels.setPixelColor(j,0); // erase bottom
    }
    j=(NUMPIXELS-19);
    k=0;
    while ((j<NUMPIXELS) && (mystr[k])) {
      if (pgm_read_byte_near(font1 + (ConvChar(mystr[k])*3)) & m) {pixels.setPixelColor(j,c);}
      j++;
      if (pgm_read_byte_near(font1 + ((ConvChar(mystr[k])*3)+1)) & m) {pixels.setPixelColor(j,c);}
      j++;
      if (pgm_read_byte_near(font1 + ((ConvChar(mystr[k])*3)+2)) & m) {pixels.setPixelColor(j,c);}
      j+=2;
      k++; // next character
    }
    m*=2; // next mask
    pixels.show();
    delay(100);
  }
}

void NoLnPrint(const uint32_t c,const char* mystr) {
int i,j,k,m;
  for(j=(NUMPIXELS-8*19);j<(NUMPIXELS);j++) {
    pixels.setPixelColor(j,0); // erase bottom
  }
  m=1;
  for(i=0;i<8;i++){
    j=(NUMPIXELS-8*19) + (i*19);
    k=0;
    while ((k<5) && (mystr[k])) {
      if (pgm_read_byte_near(font1 + (ConvChar(mystr[k])*3)) & m) {pixels.setPixelColor(j,c);}
      j++;
      if (pgm_read_byte_near(font1 + ((ConvChar(mystr[k])*3)+1)) & m) {pixels.setPixelColor(j,c);}
      j++;
      if (pgm_read_byte_near(font1 + ((ConvChar(mystr[k])*3)+2)) & m) {pixels.setPixelColor(j,c);}
      j+=2;
      k++; // next character
    }
    m*=2; // next mask
  }
    pixels.show();
}

void setup() {
  // initialize the serial communication:
  Serial.begin(115200);

  delay(400);  // catch Due reset problem

  // initialize the SD card
  if (!card.begin(CHIP_SELECT)) error("card.begin");
  // initialize a FAT16 volume
  if (!Fat16::init(&card)) error("Fat16::init");
  
  pinMode(2, INPUT); // fire button
  pixels.begin(); // This initializes the NeoPixel library.

  //pixels.updateLength(1); // To save some mem, short neopixels
  //ReadLevel(2);
  //pixels.updateLength(NUMPIXELS); // restore neopixels
  cls();
  //Serial.print(F("Level: "));
  LnPrint(0x000A00,"Level");
  LnPrint(0x0A0A00,"1");
}

uint32_t dravchar(char ch) {
  /*switch (ch) {
    case ' ': Serial.print(F("\033[40m ")); break; // empty floor
    case '#': Serial.print(F("\033[0;41;37m#")); break; // wall 
    case '$': Serial.print(F("\033[0;43;30m#")); break; // box
    case '.': Serial.print(F("\033[44;34;1mo")); break; // pad
    case '@': Serial.print(F("\033[40;34;1mO")); break; // man
    case '*': Serial.print(F("\033[43;37;1m#")); break; // box on pad
    case '+': Serial.print(F("\033[44;36;1mO")); break; // man on pad
  }*/
  switch (ch) {
    case ' ': return 0x000000; break; // empty floor
    case '#': return 0x00000A; break; // wall 
    case '$': return 0x0A0000; break; // box
    case '.': return 0x060606; break; // pad
    case '@': return 0x000A00; break; // man
    case '*': return 0x0F0404; break; // box on pad
    case '+': return 0x040F04; break; // man on pad
  }

}

char GetKey() { // read key/joystick Up=A Down=B Right=C Left=D
char tmpch;
// ansi arrows Up \033[A Down \033[B Right \033[C Left \33[D
/*  do {
    if (Serial.available() > 0) {
     // read the oldest byte in the serial buffer:
     tmpch = Serial.read();
    }
  } while ((tmpch<'A') || (tmpch>'D'));*/
  byte Keyboard = (PIND & 0xF0); // read joystick
  while (Keyboard) {
    delay(10);
    Keyboard = (PIND & 0xF0); // read joystick
  }
  delay(80);
  while (!Keyboard) {
    delay(10);
    Keyboard = (PIND & 0xF4); // read joystick
  }
// ansi arrows Up \033[A Down \033[B Right \033[C Left \33[D
  if (Keyboard & 4)
  {
    tmpch = 'E'; // fire
  }
  else if (Keyboard & 16)
  {
    tmpch = 'C'; // right
  }
  else if (Keyboard & 32)
  {
    tmpch = 'D'; // left
  }
  else if (Keyboard & 128)
  {
    tmpch = 'A'; // up
  }
  else
  {
    tmpch = 'B'; // down
  }
  return tmpch;
}

void PrintStats(const byte fin) {
  char mytext[6];
  static const char StepsM[] PROGMEM = "Steps";
  static const char PushsM[] PROGMEM = "Pushs";
  static const char BoxesM[] PROGMEM = "Boxes";
  static const char OnPosM[] PROGMEM = "OnPos";
  static const char EmptM[] PROGMEM = "";
  static const char GoodM[] PROGMEM = "Good";
  static const char JobM[] PROGMEM = "Job";
  static const char LevelM[] PROGMEM = "Level";
  static const char JednaM[] PROGMEM = "1";
  if (fin) {
          strcpy_P(mytext,GoodM);
          LnPrint(0x100000,mytext);
          strcpy_P(mytext,JobM);
          LnPrint(0x100000,mytext);
  }
          strcpy_P(mytext,StepsM);
          LnPrint(0x000A00,mytext);
          sprintf(mytext,"%d",Steps);
          LnPrint(0x0A0A00,mytext);
          strcpy_P(mytext,PushsM);
          LnPrint(0x000A00,mytext);
          sprintf(mytext,"%d",Pushes);
          LnPrint(0x0A0A00,mytext);
          strcpy_P(mytext,BoxesM);
          LnPrint(0x000A00,mytext);
          sprintf(mytext,"%d",Boxes);
          LnPrint(0x0A0A00,mytext);
          strcpy_P(mytext,OnPosM);
          LnPrint(0x000A00,mytext);
          sprintf(mytext,"%d",BoPs);
          LnPrint(0x0A0A00,mytext);
  if (fin) {
         strcpy_P(mytext,LevelM);
         LnPrint(0x000A00,mytext);
         strcpy_P(mytext,JednaM);
         LnPrint(0x0A0A00,mytext);
  } else {
          strcpy_P(mytext,EmptM);
          LnPrint(0x000A00,mytext);
  }
}

void loop() {
  if (!GameState) {
  char mytext[4];
    //cls2();
    sprintf(mytext,"%d",Level);
    NoLnPrint(0x0A0A00,mytext);
    //Serial.println(Level);
    incomingByte = GetKey();
    if (incomingByte == 'A') { // up
      Level++;
      if (!Level) Level++;
    }
    if (incomingByte == 'B') { // down
      Level--;
      if (!Level) Level--;
    }
    if ((incomingByte == 'C') || (incomingByte == 'E')) { // Right
      //pixels.updateLength(10); // To save some mem, short neopixels
      ReadLevel(Level);
      //pixels.updateLength(NUMPIXELS); // restore neopixels
      GameState=1;
    }
  } else {
    // send the value of analog input 0:
    Boxes=0; BoPs=0;
    for (i = 0; i < MaxY; i++){
      for (j = 0; j < MaxX; j++){
       if (level[i][j] == '$') {Boxes++;};
       if (level[i][j] == '*') {BoPs++;};
       }
     }
    cls();
    /*Serial.print(F("Steps: "));
    Serial.println(Steps);
    Serial.print(F("Pushes: "));
    Serial.println(Pushes);
    Serial.print(F("Boxes \033[0;43;30m#\033[0m: "));
    Serial.print(Boxes);
    Serial.print(F("  Boxes on pad \033[43;37;1m#\033[0m: "));
    Serial.println(BoPs); */
    for (i = 0; i < MaxY; i++){
      for (j = 0; j < MaxX; j++){
       pixels.setPixelColor(j + (i*19), dravchar(level[i][j]));
       }
       //Serial.println();
     }
    pixels.show(); 
    if (!Boxes) {GameState=0;
    PrintStats(1);
    //Serial.println(F("Good yob!!!"));
    }
    //Serial.print(F("FreeMem:"));
    Serial.println(FreeRam());
    
    do {
    //itn=Serial.read(); // -1 = no data, else character.
      Draw=0;
       incomingByte = GetKey();
       // Ansi = Up \033[A Down \033[B Right \033[C Left \33[D
       // if it's a capital A it mean Up:
        if (incomingByte == 'A') {
          if ((PosY>0) && (level[PosY][PosX]=='@')) {
            switch (level[PosY-1][PosX]) {
              case ' ': // floor
                level[PosY--][PosX]=' ';
                level[PosY][PosX]='@';
                Draw=1;
                break;
              case '.': // pad
                level[PosY--][PosX]=' ';
                level[PosY][PosX]='+';
                Draw=1;
                break;
              case '$': // box
                if ((PosY>1) && (level[PosY-2][PosX]==' ')) {
                  level[PosY-2][PosX]='$';
                  level[PosY--][PosX]=' ';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                if ((PosY>1) && (level[PosY-2][PosX]=='.') && (Draw==0)) {
                  level[PosY-2][PosX]='*';
                  level[PosY--][PosX]=' ';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                break;
              case '*': // box on pad
                if ((PosY>1) && (level[PosY-2][PosX]==' ')) {
                  level[PosY-2][PosX]='$';
                  level[PosY--][PosX]=' ';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                if ((PosY>1) && (level[PosY-2][PosX]=='.') && (Draw==0)) {
                  level[PosY-2][PosX]='*';
                  level[PosY--][PosX]=' ';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                break;
            }
          }
          if ((PosY>0) && (level[PosY][PosX]=='+') && (Draw==0)) {
            switch (level[PosY-1][PosX]) {
              case ' ': // floor
                level[PosY--][PosX]='.';
                level[PosY][PosX]='@';
                Draw=1;
                break;
              case '.': // pad
                level[PosY--][PosX]='.';
                level[PosY][PosX]='+';
                Draw=1;
                break;
              case '$': // box
                if ((PosY>1) && (level[PosY-2][PosX]==' ')) {
                  level[PosY-2][PosX]='$';
                  level[PosY--][PosX]='.';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                if ((PosY>1) && (level[PosY-2][PosX]=='.') && (Draw==0)) {
                  level[PosY-2][PosX]='*';
                  level[PosY--][PosX]='.';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                break;
              case '*': // box on pad
                if ((PosY>1) && (level[PosY-2][PosX]==' ')) {
                  level[PosY-2][PosX]='$';
                  level[PosY--][PosX]='.';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                if ((PosY>1) && (level[PosY-2][PosX]=='.') && (Draw==0)) {
                  level[PosY-2][PosX]='*';
                  level[PosY--][PosX]='.';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                break;
            }
          }
        } // Incomming byte
        if (incomingByte == 'B') {
          if ((PosY<MaxY-1) && (level[PosY][PosX]=='@')) {
            switch (level[PosY+1][PosX]) {
              case ' ': // floor
                level[PosY++][PosX]=' ';
                level[PosY][PosX]='@';
                Draw=1;
                break;
              case '.': // pad
                level[PosY++][PosX]=' ';
                level[PosY][PosX]='+';
                Draw=1;
                break;
              case '$': // box
                if ((PosY<MaxY-2) && (level[PosY+2][PosX]==' ')) {
                  level[PosY+2][PosX]='$';
                  level[PosY++][PosX]=' ';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                if ((PosY<MaxY-2) && (level[PosY+2][PosX]=='.') && (Draw==0)) {
                  level[PosY+2][PosX]='*';
                  level[PosY++][PosX]=' ';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                break;
              case '*': // box on pad
                if ((PosY<MaxY-2) && (level[PosY+2][PosX]==' ')) {
                  level[PosY+2][PosX]='$';
                  level[PosY++][PosX]=' ';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                if ((PosY<MaxY-2) && (level[PosY+2][PosX]=='.') && (Draw==0)) {
                  level[PosY+2][PosX]='*';
                  level[PosY++][PosX]=' ';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                break;
            }
          }
          if ((PosY<MaxY-1) && (level[PosY][PosX]=='+') && (Draw==0)) {
            switch (level[PosY+1][PosX]) {
              case ' ': // floor
                level[PosY++][PosX]='.';
                level[PosY][PosX]='@';
                Draw=1;
                break;
              case '.': // pad
                level[PosY++][PosX]='.';
                level[PosY][PosX]='+';
                Draw=1;
                break;
              case '$': // box
                if ((PosY<MaxY-2) && (level[PosY+2][PosX]==' ')) {
                  level[PosY+2][PosX]='$';
                  level[PosY++][PosX]='.';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                if ((PosY<MaxY-2) && (level[PosY+2][PosX]=='.') && (Draw==0)) {
                  level[PosY+2][PosX]='*';
                  level[PosY++][PosX]='.';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                break;
              case '*': // box on pad
                if ((PosY<MaxY-2) && (level[PosY+2][PosX]==' ')) {
                  level[PosY+2][PosX]='$';
                  level[PosY++][PosX]='.';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                if ((PosY<MaxY-2) && (level[PosY+2][PosX]=='.') && (Draw==0)) {
                  level[PosY+2][PosX]='*';
                  level[PosY++][PosX]='.';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                break;
            }
          }
        } // Incomming byte
        if (incomingByte == 'C') {
          if ((PosX<MaxX-1) && (level[PosY][PosX]=='@')) {
            switch (level[PosY][PosX+1]) {
              case ' ': // floor
                level[PosY][PosX++]=' ';
                level[PosY][PosX]='@';
                Draw=1;
                break;
              case '.': // pad
                level[PosY][PosX++]=' ';
                level[PosY][PosX]='+';
                Draw=1;
                break;
              case '$': // box
                if ((PosX<MaxX-2) && (level[PosY][PosX+2]==' ')) {
                  level[PosY][PosX+2]='$';
                  level[PosY][PosX++]=' ';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                if ((PosX<MaxX-2) && (level[PosY][PosX+2]=='.') && (Draw==0)) {
                  level[PosY][PosX+2]='*';
                  level[PosY][PosX++]=' ';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                break;
              case '*': // box on pad
                if ((PosX<MaxX-2) && (level[PosY][PosX+2]==' ')) {
                  level[PosY][PosX+2]='$';
                  level[PosY][PosX++]=' ';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                if ((PosX<MaxX-2) && (level[PosY][PosX+2]=='.') && (Draw==0)) {
                  level[PosY][PosX+2]='*';
                  level[PosY][PosX++]=' ';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                break;
            }
          }
          if ((PosX<MaxX-1) && (level[PosY][PosX]=='+') && (Draw==0)) {
            switch (level[PosY][PosX+1]) {
              case ' ': // floor
                level[PosY][PosX++]='.';
                level[PosY][PosX]='@';
                Draw=1;
                break;
              case '.': // pad
                level[PosY][PosX++]='.';
                level[PosY][PosX]='+';
                Draw=1;
                break;
              case '$': // box
                if ((PosX<MaxX-2) && (level[PosY][PosX+2]==' ')) {
                  level[PosY][PosX+2]='$';
                  level[PosY][PosX++]='.';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                if ((PosX<MaxX-2) && (level[PosY][PosX+2]=='.') && (Draw==0)) {
                  level[PosY][PosX+2]='*';
                  level[PosY][PosX++]='.';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                break;
              case '*': // box on pad
                if ((PosX<MaxX-2) && (level[PosY][PosX+2]==' ')) {
                  level[PosY][PosX+2]='$';
                  level[PosY][PosX++]='.';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                if ((PosX<MaxX-2) && (level[PosY][PosX+2]=='.') && (Draw==0)) {
                  level[PosY][PosX+2]='*';
                  level[PosY][PosX++]='.';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                break;
            }
          }
        } // Incomming byte
        if (incomingByte == 'D') {
          if ((PosX>0) && (level[PosY][PosX]=='@')) {
            switch (level[PosY][PosX-1]) {
              case ' ': // floor
                level[PosY][PosX--]=' ';
                level[PosY][PosX]='@';
                Draw=1;
                break;
              case '.': // pad
                level[PosY][PosX--]=' ';
                level[PosY][PosX]='+';
                Draw=1;
                break;
              case '$': // box
                if ((PosX>1) && (level[PosY][PosX-2]==' ')) {
                  level[PosY][PosX-2]='$';
                  level[PosY][PosX--]=' ';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                if ((PosX>1) && (level[PosY][PosX-2]=='.') && (Draw==0)) {
                  level[PosY][PosX-2]='*';
                  level[PosY][PosX--]=' ';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                break;
              case '*': // box on pad
                if ((PosX>1) && (level[PosY][PosX-2]==' ')) {
                  level[PosY][PosX-2]='$';
                  level[PosY][PosX--]=' ';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                if ((PosX>1) && (level[PosY][PosX-2]=='.') && (Draw==0)) {
                  level[PosY][PosX-2]='*';
                  level[PosY][PosX--]=' ';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                break;
            }
          }
          if ((PosX>0) && (level[PosY][PosX]=='+') && (Draw==0)) {
            switch (level[PosY][PosX-1]) {
              case ' ': // floor
                level[PosY][PosX--]='.';
                level[PosY][PosX]='@';
                Draw=1;
                break;
              case '.': // pad
                level[PosY][PosX--]='.';
                level[PosY][PosX]='+';
                Draw=1;
                break;
              case '$': // box
                if ((PosX>1) && (level[PosY][PosX-2]==' ')) {
                  level[PosY][PosX-2]='$';
                  level[PosY][PosX--]='.';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                if ((PosX>1) && (level[PosY][PosX-2]=='.') && (Draw==0)) {
                  level[PosY][PosX-2]='*';
                  level[PosY][PosX--]='.';
                  level[PosY][PosX]='@';
                  Draw=1;
                  Pushes++;
                }
                break;
              case '*': // box on pad
                if ((PosX>1) && (level[PosY][PosX-2]==' ')) {
                  level[PosY][PosX-2]='$';
                  level[PosY][PosX--]='.';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                if ((PosX>1) && (level[PosY][PosX-2]=='.') && (Draw==0)) {
                  level[PosY][PosX-2]='*';
                  level[PosY][PosX--]='.';
                  level[PosY][PosX]='+';
                  Draw=1;
                  Pushes++;
                }
                break;
            }
          }
        } // Incomming byte
        if (incomingByte == 'E') {
          Draw=1;
          PrintStats(0);
          Steps--;
        } 
    } while (Draw==0);
    Steps++;
    // wait a bit for the analog-to-digital converter
    // to stabilize after the last reading:
    //delay(200);
  }
}

